#version 330

struct BaseLight {
	vec3 color;
	float intensity;
};

struct DirectionalLight {
	BaseLight base;
	vec3 position;
};

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

uniform sampler2D gPositionMap;
uniform sampler2D gColorMap;
uniform sampler2D gNormalMap;

uniform sampler2D depthMap;

uniform sampler2D opacityMap;

uniform vec3 lightPos;

uniform mat4 lightSpaceMatrix;

uniform DirectionalLight gDirectionalLight;
uniform vec3 gEyeWorldPos;
uniform vec2 gScreenSize;



float ShadowCalculation(vec4 fragPosLightSpace, vec2 TexCoord)
{

    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;

    float closestDepth = texture(depthMap, projCoords.xy).r; 
    float currentDepth = projCoords.z;

    vec3 normal = normalize(texture(gNormalMap, TexCoord).xyz);
    vec3 lightDir = normalize(lightPos - texture(gPositionMap, TexCoord).xyz);
    float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
    
    // PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(depthMap, 0);
    for(int x = -1; x <= 1; ++x) {
        for(int y = -1; y <= 1; ++y) {
            float pcfDepth = texture(depthMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    
    // outside the far_plane
    if(projCoords.z > 1.0) {
        shadow = 0.0;
	}
        
    return shadow;
}


void main() {
	vec2 TexCoord = gl_FragCoord.xy / gScreenSize;

	vec3 color = texture(gColorMap, TexCoord).rgb;
    vec3 normal = normalize(texture(gNormalMap, TexCoord).xyz);
    vec3 lightColor = gDirectionalLight.base.color * gDirectionalLight.base.intensity;

    // Diffuse
    vec3 lightDir = normalize(lightPos - texture(gPositionMap, TexCoord).xyz);
    float diff = max(dot(lightDir, normal), 0.0);
    vec3 diffuse = diff * lightColor;

    // Specular
    vec3 viewDir = normalize(gEyeWorldPos - texture(gPositionMap, TexCoord).xyz);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = 0.0;
    vec3 halfwayDir = normalize(lightDir + viewDir);  
    spec = pow(max(dot(normal, halfwayDir), 0.0), 64.0);
    vec3 specular = spec * lightColor;    
    
	// Calculate shadow
	vec4 FragPosLightSpace = lightSpaceMatrix * vec4(texture(gPositionMap, TexCoord).xyz, 1.0);

	float opacityFactor = 1.0f;
	if(texture(opacityMap, TexCoord).x < 1.0f) {
           opacityFactor = (texture(opacityMap, TexCoord).x / 2);
	}
	float shadow = ShadowCalculation(FragPosLightSpace, TexCoord) * opacityFactor;
    
	// final lighting
	vec3 lighting = ((1.0 - shadow) * (diffuse + specular)) * color * texture(opacityMap, TexCoord).x;    
    
	FragColor = vec4(lighting, 1.0f);
	/*if(shadow > 0.0f) {
		BrightColor = vec4(lighting,1.0f);
	}*/
	
	//bloom?
	float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 0.7) {
		BrightColor = vec4(FragColor.rgb, 1.0);
	}
	else {
			BrightColor = vec4(0,0,0,1);
		}
	
	
}